/*
===============================================================================

 Casio Basic RUNTIME library for fx-9860G series      v0.01

 copyright(c)2015 by sentaro21
 e-mail sentaro21@pm.matrix.jp

===============================================================================
*/
#include <ctype.h>
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <timer.h>
#include "Fkey.h"
#include "inp.h"
#include "CB_lib.h"

//-----------------------------------------------------------------------------
// Casio Basic inside
//-----------------------------------------------------------------------------
int BusyInd     = 1;	// BusyInd=0: running indicator off,  BusyInd=1: on

int	DrawType    = 1;	// 0:connect  1:Plot
int	Coord       = 1;	// 0:off 1:on
int	Grid        = 1;	// 0:off 1:on
int	Axes        = 1;	// 0:off 1:on
int	Label       = 1;	// 0:off 1:on
int	Derivative  = 1;	// 0:off 1:on
int S_L_Style   = S_L_Normal;

double Previous_X=1e308 ;	// ViewWindow Previous X
double Previous_Y=1e308 ;	// ViewWindow Previous Y

//-----------------------------------------------------------------------------
// Casio Basic Gloval variable
//-----------------------------------------------------------------------------
double  REG[37];

double	Xfct     =  2;					// 
double	Yfct     =  2;					// 

double	traceAry[130];		// Graph array X

//------------------------------------------------------------------------------
double MOD(double numer, double denom) {
	return ( fmod( numer, denom ) );
}
//------------------------------------------------------------------------------

void Text(int y, int x, char*str) {
	PrintMini(  x, y, (unsigned char*)str,MINI_OVER);
}

//------------------------------------------------------------------------------
void DrawBusy()		// BusyInd=0: running indicator off,  BusyInd=1: on
{
	unsigned char BusyDATA[]={ 0xff,0xff,0xff,0xff };
	DISPGRAPH Gbsy;

    if ( BusyInd == 0 ) return;
    Gbsy.x = 124; 
    Gbsy.y =   0; 
    Gbsy.GraphData.width =  4;
    Gbsy.GraphData.height = 4;
    
    Gbsy.GraphData.pBitmap = BusyDATA; 	// Busy pattern
    Gbsy.WriteModify = IMB_WRITEMODIFY_NORMAL; 
    Gbsy.WriteKind = IMB_WRITEKIND_OR;
    Bdisp_WriteGraph_DD(&Gbsy); 		// drawing only display driver
}
//----------------------------------------------------------------------------------------------
int VWtoPXY(double x, double y, int *px, int *py){	// ViewWwindow(x,y) -> pixel(x,y)
	if ( ( Xdot == 0 ) || ( Ydot == 0 ) ) return -1;
	if ( Xmax == Xmin )		return -1;
	if ( Xmax >  Xmin )		*px =       (x-Xmin)/Xdot  +1.5 ;
	if ( Xmax <  Xmin )		*px = 126 - (x-Xmax)/-Xdot +1.49999999999999 ;
	if ( Ymax == Ymin )		return -1;
	if ( Ymax >  Ymin )		*py =  62 - (y-Ymin)/Ydot  +1.49999999999999 ;
	if ( Ymax <  Ymin )		*py =       (y-Ymax)/-Ydot +1.5 ;
	if ( (*px<1) || (*px>127) ) return -1 ;
	if ( (*py<1) || (*py> 63) ) return -1 ;
	return 0;
}
void PXYtoVW(int px, int py, double *x, double *y){	// pixel(x,y) -> ViewWwindow(x,y)
	if ( Xmax == Xmin )		*x = Xmin ;
	if ( Xmax >  Xmin )		*x = (    (double)px-1)*Xdot  + Xmin ;
	if ( Xmax <  Xmin )		*x = (126-(double)px+1)*-Xdot + Xmax ;
	if ( Ymax == Ymin )		*y = Ymin ;
	if ( Ymax >  Ymin )		*y = ( 62-(double)py+1)*Ydot  + Ymin ;
	if ( Ymax <  Ymin )		*y = (    (double)py-1)*-Ydot + Ymax ;
	if ( fabs(*x) < 1.0e-13 ) *x=0;	// zero adjust
	if ( fabs(*y) < 1.0e-13 ) *y=0;	// zero adjust
}

void PlotGrid(double x, double y){
	int px,py;
	if ( ( x==0 ) && ( y==0 ) ) return;
	if ( VWtoPXY( x,y, &px, &py) ) return;
	if ( (px<1) || (px>127) ) return ;
	if ( (py<1) || (py> 63) ) return ;
	if ( x==0 ) { Bdisp_SetPoint_VRAM( px+1, py  , 1); return; }
	if ( y==0 )	{ Bdisp_SetPoint_VRAM( px  , py-1, 1); return; }
	Bdisp_SetPoint_VRAM( px, py, 1);
}

void ViewWindow( double xmin, double xmax, double xscl, double ymin, double ymax, double yscl){
	double x,y;
	
	Xmin  =xmin;
	Xmax  =xmax;
	Xscl=xscl;
	Xdot  =(Xmax-Xmin)/126.0;
	Ymin  =ymin;
	Ymax  =ymax;
	Yscl=yscl;
	Ydot  =(Ymax-Ymin)/ 62.0;
	
	Bdisp_AllClr_VRAM();			// ------ Clear VRAM 

	if ( Axes ) {
		Horizontal(0, S_L_Normal);
		Vertical(0, S_L_Normal);
	}

	if ( Grid == 0 ) {
			if ( Yscl > 0 ) {
				if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(0,y);
				if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(0,y);
				if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(0,y);
				if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(0,y);
			}
			if ( Xscl > 0 ) {
				if ( Xmin <= 0 ) for ( x=0; x>=Xmin; x-=Xscl )	PlotGrid(x,0);
				if ( Xmin >  0 ) for ( x=0; x<=Xmin; x+=Xscl )	PlotGrid(x,0);
				if ( Xmax <= 0 ) for ( x=0; x>=Xmax; x-=Xscl )	PlotGrid(x,0);
				if ( Xmax >  0 ) for ( x=0; x<=Xmax; x+=Xscl )	PlotGrid(x,0);
			}
	}	
	if ( Grid ) {
		if ( Xscl == 0 ) {
			if ( Yscl > 0 ) {
				if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(0,y);
				if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(0,y);
				if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(0,y);
				if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(0,y);
			}
		}	
		if ( Yscl == 0 ) {
			if ( Xscl > 0 ) {
				if ( Xmin <= 0 ) for ( x=0; x>=Xmin; x-=Xscl )	PlotGrid(x,0);
				if ( Xmin >  0 ) for ( x=0; x<=Xmin; x+=Xscl )	PlotGrid(x,0);
				if ( Xmax <= 0 ) for ( x=0; x>=Xmax; x-=Xscl )	PlotGrid(x,0);
				if ( Xmax >  0 ) for ( x=0; x<=Xmax; x+=Xscl )	PlotGrid(x,0);
			}
		}	
		if ( ( Xscl > 0 ) && ( Yscl > 0 ) ) {
			if ( Xmin < 0 ) { 
					for ( x=0; x>=Xmin; x-=Xscl ) {
						if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(x,y);
						if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(x,y);
						if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(x,y);
						if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(x,y);
					}
			}
			if ( Xmin >= 0 ) {
					for ( x=0; x<=Xmin; x+=Xscl) {
						if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(x,y);
						if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(x,y);
						if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(x,y);
						if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(x,y);
					}
			}
			if ( Xmax < 0 ) {
					for ( x=0; x>=Xmax; x-=Xscl) {
						if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(x,y);
						if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(x,y);
						if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(x,y);
						if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(x,y);
					}
			}
			if ( Xmax >= 0 ) {
					for ( x=0; x<=Xmax; x+=Xscl) {
						if ( Ymin <= 0 ) for ( y=0; y>=Ymin; y-=Yscl )	PlotGrid(x,y);
						if ( Ymin >  0 ) for ( y=0; y<=Ymin; y+=Yscl )	PlotGrid(x,y);
						if ( Ymax <= 0 ) for ( y=0; y>=Ymax; y-=Yscl )	PlotGrid(x,y);
						if ( Ymax >  0 ) for ( y=0; y<=Ymax; y+=Yscl )	PlotGrid(x,y);
					}
			}
		}
	}

	if ( Label ) {
		PrintMini(  1,17,(unsigned char*)" ",MINI_OVER);
		PrintMini(  1,17,(unsigned char*)"Y",MINI_OVER);
		PrintMini(124,43,(unsigned char*)" ",MINI_OVER);
		PrintMini(124,43,(unsigned char*)"X",MINI_OVER);
	}
	Bdisp_PutDisp_DD();
	Previous_X=1e308; Previous_Y=1e308;	// ViewWindow Previous XY
	regX =fabs(Xmin+Xmax)/2; regY =fabs(Ymin+Ymax)/2;	// ViewWindow Current  XY
}


void ZoomIn(){
	double c,y,dx,dy,cx,cy;

	dx   = (Xmax+Xmin)/2-regX;
	Xmin = Xmin-dx;	// move center
	Xmax = Xmax-dx;
	
	cx   = (Xmax+Xmin)/2;	//
	dx   = (Xmax-Xmin)/Xfct/2;	// zoom in dx
	Xmin = cx-dx;
	Xmax = cx+dx;
	Xdot = (Xmax-Xmin)/126.0;
	
	dy   = (Ymax+Ymin)/2-regY;	// move center
	Ymin = Ymin-dy;
	Ymax = Ymax-dy;
	
	cy   = (Ymax+Ymin)/2;	// 
	dy   = (Ymax-Ymin)/Yfct/2;	// zoom in dy
	Ymin = cy-dy;
	Ymax = cy+dy;
	Ydot = (Ymax-Ymin)/ 62.0;
	
	regX = (Xmax+Xmin)/2; // center
	regY = (Ymax+Ymin)/2; // center
}

void ZoomOut(){
	double dx,dy,cx,cy;
	
	dx   = (Xmax+Xmin)/2-regX;
	Xmin = Xmin-dx;	// move center
	Xmax = Xmax-dx;
	
	cx   = (Xmax+Xmin)/2;	//
	dx   = (Xmax-Xmin)*Xfct/2;	// zoom out dx
	Xmin = cx-dx;
	Xmax = cx+dx;
	Xdot = (Xmax-Xmin)/126.0;
	
	dy   = (Ymax+Ymin)/2-regY;	// move center
	Ymin = Ymin-dy;
	Ymax = Ymax-dy;
	
	cy   = (Ymax+Ymin)/2;	// 
	dy   = (Ymax-Ymin)*Yfct/2;	// zoom out dy
	Ymin = cy-dy;
	Ymax = cy+dy;
	Ydot = (Ymax-Ymin)/ 62.0;
	
	regX = (Xmax+Xmin)/2; // center
	regY = (Ymax+Ymin)/2; // center
}

//------------------------------------------------------------------------------ PLOT
void Plotsub(double x, double y, int kind, int mode){
	int px,py;
	if ( VWtoPXY( x,y, &px, &py) == 0) {;
		switch (mode) {
			case 1:
				Bdisp_SetPoint_VRAM( px, py, kind);
				break;
			case 2:
				Bdisp_SetPoint_DD( px, py, kind);
				break;
			case 3:
				Bdisp_SetPoint_DDVRAM( px, py , kind);
				break;
		}		
	}
	regX=x; regY=y;
}

void PlotOn_VRAM(double x, double y){
	Plotsub( x, y, 1, 1);		// VRAM
}
void PlotOn_DD(double x, double y){
	Plotsub( x, y, 1, 2);		// DD
}
void PlotOn_DDVRAM(double x, double y){
	Plotsub( x, y, 1, 3);		// DDVRAM
}
void PlotOff_VRAM(double x, double y){
	Plotsub( x, y, 0, 1);		// VRAM
}
void PlotOff_DD(double x, double y){
	Plotsub( x, y, 0, 2);		// DD
}
void PlotOff_DDVRAM(double x, double y){
	Plotsub( x, y, 0, 3);		// DDVRAM
}
void PlotChg_VRAM(double x,  double y){
	int px,py;
	if ( VWtoPXY( x,y, &px, &py) ) return;
	if ( Bdisp_GetPoint_VRAM(px, py) )
		 Bdisp_SetPoint_VRAM(px, py, 0);
	else Bdisp_SetPoint_VRAM(px, py, 1);
}

void PlotChg_DDVRAM(double x, double y){
	PlotChg_VRAM(x, y);
	Bdisp_PutDisp_DD();
}
void PxlOn_VRAM(int py, int px){
	Bdisp_SetPoint_VRAM(px, py, 1);
}
void PxlOn_DD(int py, int px){
	Bdisp_SetPoint_DD(px, py, 1);
}
void PxlOn_DDVRAM(int py, int px){
	Bdisp_SetPoint_DDVRAM(px, py, 1);
}
void PxlOff_VRAM(int py, int px){
	Bdisp_SetPoint_VRAM(px, py, 0);
}
void PxlOff_DD(int py, int px){
	Bdisp_SetPoint_DD(px, py, 0);
}
void PxlOff_DDVRAM(int py, int px){
	Bdisp_SetPoint_DDVRAM(px, py, 0);
}
int PxlTest(int py, int px) {
	return	Bdisp_GetPoint_VRAM(px, py);
}

//------------------------------------------------------------------------------ LINE
void LinesubSetPoint(int px, int py) {
	if ( ( px <   1 ) || ( px > 127 ) || ( py <   1 ) || ( py >  63 ) ) return;
	Bdisp_SetPoint_VRAM(px, py, 1);
}

void Linesub(int px1, int py1, int px2, int py2, int style) {
	int i, j;
	int x, y;
	int dx, dy; // delta x,y
	int wx, wy; // width x,y
	int Styleflag=1;
	int tmp;

	if (px1==px2) { dx= 0; wx=0; }
	if (px1< px2) { dx= 1; wx=px2-px1; }
	if (px1> px2) { dx=-1; wx=px1-px2;}
	if (py1==py2) { dy= 0; wy=0; }
	if (py1< py2) { dy= 1; wy=py2-py1; }
	if (py1> py2) { dy=-1; wy=py1-py2; }

	tmp=S_L_Style;
	if ( style >=0 ) S_L_Style=style;

	switch (S_L_Style) {
		case S_L_Normal:	// ---------- Normal
			if (wx==0) {	// vertical line
					x=px1; y=py1;
					while( wy>=0 ) {
						LinesubSetPoint(x, y);
						wy--; y+=dy;
					}
			}
			if (wy==0) { // horizontal line
					x=px1; y=py1;
					while( wx>=0 ) {
						LinesubSetPoint(x, y);
						wx--; x+=dx;
					}
			}
			
			if (wx>=wy) {
				if (dy>0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						LinesubSetPoint(x, y);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y++; }
					}
				}
				if (dy<0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						LinesubSetPoint(x, y);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y--; }
					}
				}
				
			} else {
				if (dx>0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						LinesubSetPoint(x, y);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x++; }
					}
				}
				if (dx<0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						LinesubSetPoint(x, y);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x--; }
					}
				}
				
			}
			break;
		case S_L_Dot:		// ---------- Dot
			if (wx==0) {	// vertical line
					x=px1; y=py1;
					while( wy>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wy--; y+=dy;
						Styleflag = 1-Styleflag;
					}
			}
			if (wy==0) { // horizontal line
					x=px1; y=py1;
					while( wx>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wx--; x+=dx;
						Styleflag = 1-Styleflag;
					}
			}
			
			if (wx>wy) {
				if (dy>0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y++; }
						Styleflag = 1-Styleflag;
					}
				}
				if (dy<0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y--; }
						Styleflag = 1-Styleflag;
					}
				}
				
			} else {
				if (dx>0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x++; }
						Styleflag = 1-Styleflag;
					}
				}
				if (dx<0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						if (Styleflag)
						LinesubSetPoint(x, y);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x--; }
						Styleflag = 1-Styleflag;
					}
				}
				
			}
			break;
		case S_L_Thick:	// ---------- Thick
			if (wx==0) {	// vertical line
					x=px1; y=py1;
					while( wy>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wy--; y+=dy;
					}
			}
			if (wy==0) { // horizontal line
					x=px1; y=py1;
					while( wx>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wx--; x+=dx;
					}
			}
			
			if (wx>=wy) {
				if (dy>0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y++; }
					}
				}
				if (dy<0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y--; }
					}
				}
				
			} else {
				if (dx>0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x++; }
					}
				}
				if (dx<0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						LinesubSetPoint(x  , y  );
						LinesubSetPoint(x  , y-1);
						LinesubSetPoint(x-1, y  );
						LinesubSetPoint(x-1, y-1);
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x--; }
					}
				}
				
			}
			break;
		case S_L_Broken:	// ---------- Broken
			if (wx==0) {	// vertical line
					x=px1; y=py1;
					while( wy>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wy--; y+=dy;
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
			}
			if (wy==0) { // horizontal line
					x=px1; y=py1;
					while( wx>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wx--; x+=dx;
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
			}
			
			if (wx>=wy) {
				if (dy>0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y++; }
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
				}
				if (dy<0) {
					x=px1; y=py1; j=wx/2; i=wx;
					while( wx>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wx--; x+=dx;
						j-=wy; if (j<0) { j+=i; y--; }
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
				}
				
			} else {
				if (dx>0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x++; }
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
				}
				if (dx<0) {
					x=px1; y=py1; j=wy/2; i=wy;
					while( wy>=0 ) {
						if (Styleflag==1) {
							LinesubSetPoint(x  , y  );
							LinesubSetPoint(x  , y-1);
							LinesubSetPoint(x-1, y  );
							LinesubSetPoint(x-1, y-1);
						}
						wy--; y+=dy;
						j-=wx; if (j<0) { j+=i; x--; }
						Styleflag++; if (Styleflag>2) Styleflag=0;
					}
				}
				
			}
			break;
		default:
			break;
		}
	S_L_Style=tmp;
}


void Line(int style) {
	int px1,px2,py1,py2;
	if ( Previous_X > 1e307 ) { 
		 Previous_X = regX;
		 Previous_Y = regY;
		return;
	}
	if ( VWtoPXY( Previous_X, Previous_Y, &px1, &py1) ) return;
	if ( VWtoPXY(       regX,       regY, &px2, &py2) ) return;
	Linesub( px1, py1, px2, py2, style);
	Previous_X = regX;
	Previous_Y = regY;
}

void F_Line(double x1, double y1, double x2, double y2, int style) {
	int px1,px2,py1,py2;
	int tmp;
	if ( VWtoPXY( x1, y1, &px1, &py1) ) return;
	if ( VWtoPXY( x2, y2, &px2, &py2) ) return;
	Linesub( px2, py2, px1, py1 ,style);
}

void Vertical(double x, int style) {
	int px,py;
	if ( VWtoPXY( x, 0, &px, &py) ) return;
	Linesub( px, 1, px, 63, style);
}
void Horizontal(double y, int style) {
	int px,py;
	if ( VWtoPXY( 0, y, &px, &py) ) return;
	Linesub( 1, py, 127, py, style);
}
//------------------------------------------------------------------------------
int GCursorflag = 1;	// GCursor Pixel ON:1 OFF:0
int GCursorX;
int GCursorY;

void GCursorFlashing()		// timer IRQ handler
{
	if ( ( 0<GCursorX ) && ( GCursorX<128 ) && ( 0<GCursorY ) && ( GCursorY<64 ) ) {
		switch (GCursorflag) {
			case 0:
				Bdisp_SetPoint_DDVRAM(GCursorX, GCursorY,1);
				GCursorflag=1;
				break;
			case 1:
				Bdisp_SetPoint_DDVRAM(GCursorX, GCursorY,0);
				GCursorflag=0;
				break;
			default:
				break;
		}
	}
}

void GCursorSetFlashMode(int set)	// 1:on  0:off
{
	switch (set) {
		case 0:
			KillTimer(ID_USER_TIMER1);
			break;
		case 1:
			SetTimer(ID_USER_TIMER1, 250, &GCursorFlashing);
			GCursorflag=1;		// graphic cursor initialize
			break;
		default:
			break;
	}
}

void DrawGCSR( int x, int y )
{
	unsigned char PlotCsrDATA[]={ 0x1C,0x14,0x77,0x41,0x77,0x14,0x1C };
	unsigned char PlotCsrMASK[]={ 0x1C,0x1C,0x7F,0x7F,0x7F,0x1C,0x1C };

    DISPGRAPH GCSR; 
    
	if ( (0<x) && (x<128) && (0<y) && (y<64) ) {
		GCSR.x = x-4; 
		GCSR.y = y-3; 
		GCSR.GraphData.width =	8; if ( x>124) GCSR.GraphData.width = 8-(x-124);
		GCSR.GraphData.height = 7;
	
		GCSR.GraphData.pBitmap = PlotCsrMASK; 	// mask pattern
		GCSR.WriteModify = IMB_WRITEMODIFY_NORMAL; 
		GCSR.WriteKind = IMB_WRITEKIND_OR;
		Bdisp_WriteGraph_VRAM(&GCSR);
	
		GCSR.WriteKind = IMB_WRITEKIND_XOR;
		Bdisp_WriteGraph_VRAM(&GCSR);
	
		GCSR.GraphData.pBitmap = PlotCsrDATA; 	// cursor pattern
		GCSR.WriteModify = IMB_WRITEMODIFY_NORMAL; 
		GCSR.WriteKind = IMB_WRITEKIND_OR;
		Bdisp_WriteGraph_VRAM(&GCSR); 
	
		Bdisp_SetPoint_VRAM(x,y,1);				// center dot
	}
	GCursorflag=1;		// graphic cursor initialize
}

//--------------------------------------------------------------
unsigned int Plot(double x, double y)
{
	int cont=1;
	char buffer[21];
	unsigned int key;
	int retcode=0;
	
	long FirstCount;	// pointer to repeat time of first repeat
	long NextCount; 	// pointer to repeat time of second repeat

	regX=x; regY=y;
	if ( VWtoPXY( regX, regY, &GCursorX, &GCursorY) ) return;	// VW(X,Y) to  graphic cursor XY
	
	Bkey_Get_RepeatTime(&FirstCount,&NextCount);	// repeat time
	Bkey_Set_RepeatTime(FirstCount,2);				// set graphic cursor repeat time  (count * 25ms)

	SaveDisp(SAVEDISP_PAGE1);	// ------ SaveDisp1
	GCursorSetFlashMode(1);	// graphic cursor flashing on
	while ( cont ) {	
		RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
		SaveDisp(SAVEDISP_PAGE1);		// ------ SaveDisp1
		PXYtoVW(GCursorX, GCursorY, &regX, &regY);	// graphic cursor XY  to  VW(X,Y)
		if ( Coord ) {
			PrintMini(  0,58,(unsigned char*)"X=",MINI_OVER);
			sprintG(buffer, regX, 13,LEFT_ALIGN); PrintMini(  8,58,(unsigned char*)buffer,MINI_OVER);
			PrintMini( 64,58,(unsigned char*)"Y=",MINI_OVER);
			sprintG(buffer, regY, 13,LEFT_ALIGN); PrintMini( 72,58,(unsigned char*)buffer,MINI_OVER);
		}
		DrawGCSR(GCursorX,GCursorY); 	// draw graphic cursor
		Bdisp_PutDisp_DD();

		GetKey(&key);
		switch (key) {
			case KEY_CTRL_EXIT:
				cont=0;
				break;
			case KEY_CTRL_EXE:
				cont=0;
				break;
			case KEY_CTRL_F3:	// setViewWindow
				if ( SetViewWindow() ) cont=0;
				break;
			case KEY_CTRL_LEFT:
				if ( GCursorX >   1 ) GCursorX--;
				break;
			case KEY_CTRL_RIGHT:
				if ( GCursorX < 127 ) GCursorX++;
				break;
			case KEY_CTRL_UP:
				if ( GCursorY >   1 ) GCursorY--;
				break;
			case KEY_CTRL_DOWN:
				if ( GCursorY <  63 ) GCursorY++;
				break;
			default:
				break;
		}
	}
	GCursorSetFlashMode(0);	// graphic cursor flashing off
	Bkey_Set_RepeatTime(FirstCount,NextCount);	// restore repeat time
	RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
	if ( retcode==0 ) Bdisp_SetPoint_VRAM( GCursorX, GCursorY, 1);
	return key ;
}

//--------------------------------------------------------------
void FkeyZoom(){
	Fkey_dispR( 1, "FACT");
	Fkey_dispN( 2, " IN");
	Fkey_dispN( 3, "OUT");
}

unsigned int Zoom_sub(unsigned int key){
	char buffer[32];
	double x,y;

	FkeyZoom();

	if (key==0) GetKey(&key);
	switch (key) {
		case KEY_CTRL_EXIT:
			break;
		case KEY_CTRL_F2:
			SetFactor();
			break;
		case KEY_CTRL_F3:
			RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
			key=ZoomXY();
			if ( key == KEY_CTRL_EXIT ) return key ;		// select zoom center 
			if ( key == KEY_CTRL_F1   ) return key ;		// trace
			ZoomIn();
			break;
		case KEY_CTRL_F4:
			RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
			key=ZoomXY();
			if ( key == KEY_CTRL_EXIT ) return key ;		// select zoom center 
			if ( key == KEY_CTRL_F1   ) return key ;		// trace
			ZoomOut();
			break;
		default:
		break;
	}
	return key; 
}

//--------------------------------------------------------------
unsigned int ZoomXY()
{
	int cont=1;
	char buffer[21];
	unsigned int key;
	int retcode=0;
	
	long FirstCount;	// pointer to repeat time of first repeat
	long NextCount; 	// pointer to repeat time of second repeat

	regX=(Xmax+Xmin)/2; regY=(Ymax+Ymin)/2;		// center
	if ( VWtoPXY( regX, regY, &GCursorX, &GCursorY) ) return;	// VW(X,Y) to  graphic cursor XY
	
	Bkey_Get_RepeatTime(&FirstCount,&NextCount);	// repeat time
	Bkey_Set_RepeatTime(FirstCount,2);				// set graphic cursor repeat time  (count * 25ms)

	SaveDisp(SAVEDISP_PAGE1);	// ------ SaveDisp1
	GCursorSetFlashMode(1);	// graphic cursor flashing on
	while ( cont ) {	
		RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
		SaveDisp(SAVEDISP_PAGE1);		// ------ SaveDisp1
		PXYtoVW(GCursorX, GCursorY, &regX, &regY);	// graphic cursor XY  to  VW(X,Y)
		if ( Coord ) {
			PrintMini(  0,58,(unsigned char*)"X=",MINI_OVER);
			sprintG(buffer, regX, 13,LEFT_ALIGN); PrintMini(  8,58,(unsigned char*)buffer,MINI_OVER);
			PrintMini( 64,58,(unsigned char*)"Y=",MINI_OVER);
			sprintG(buffer, regY, 13,LEFT_ALIGN); PrintMini( 72,58,(unsigned char*)buffer,MINI_OVER);
		}
		DrawGCSR(GCursorX,GCursorY); 	// draw graphic cursor
		Bdisp_PutDisp_DD();

		GetKey(&key);
		switch (key) {
			case KEY_CTRL_EXIT:
				cont=0;
				break;
			case KEY_CTRL_EXE:
				cont=0;
				break;
			case KEY_CTRL_F1:	// trace
				cont=0;
				break;
			case KEY_CTRL_F3:	// setViewWindow
				if ( SetViewWindow() ) cont=0;
				break;
			case KEY_CTRL_LEFT:
				if ( GCursorX >   1 ) GCursorX--;
				break;
			case KEY_CTRL_RIGHT:
				if ( GCursorX < 127 ) GCursorX++;
				break;
			case KEY_CTRL_UP:
				if ( GCursorY >   1 ) GCursorY--;
				break;
			case KEY_CTRL_DOWN:
				if ( GCursorY <  63 ) GCursorY++;
				break;
			default:
				break;
		}
	}
	GCursorSetFlashMode(0);	// graphic cursor flashing off
	Bkey_Set_RepeatTime(FirstCount,NextCount);	// restore repeat time
	RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
	return key ;
}
//--------------------------------------------------------------
int Trace(int *index )
{
	int cont=1;
	char buffer[21];
	unsigned int key;
	double dydx;
	int retcode=0;
	
	long FirstCount;	// pointer to repeat time of first repeat
	long NextCount; 	// pointer to repeat time of second repeat

	PXYtoVW(*index, 0, &regX, &regY);	// graphic cursor X  to  VW(X,dummy)
	VWtoPXY( regX, traceAry[*index], &GCursorX, &GCursorY);	// VW(X,Y) to  graphic cursor XY
	
	Bkey_Get_RepeatTime(&FirstCount,&NextCount);	// repeat time
	Bkey_Set_RepeatTime(FirstCount,2);				// set graphic cursor repeat time  (count * 25ms)

	SaveDisp(SAVEDISP_PAGE1);	// ------ SaveDisp1
	GCursorSetFlashMode(1);	// graphic cursor flashing on
	while ( cont ) {	
		RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
		SaveDisp(SAVEDISP_PAGE1);		// ------ SaveDisp1
		PXYtoVW(GCursorX, 0, &regX, &regY);	// graphic cursor X  to  VW(X,dummy)
		VWtoPXY( regX, traceAry[GCursorX], &GCursorX, &GCursorY);	// VW(X,Y) to  graphic cursor XY
		if ( Coord ) {
			sprintf(buffer, "PX=%d", GCursorX);	PrintMini(  0,0,(unsigned char*)buffer,MINI_OVER);
			sprintf(buffer, "PY=%d", GCursorY);	PrintMini( 64,0,(unsigned char*)buffer,MINI_OVER);
			PrintMini(  0,58,(unsigned char*)"X=",MINI_OVER);
			sprintG(buffer, regX,             13,LEFT_ALIGN); PrintMini(  8,58,(unsigned char*)buffer,MINI_OVER);
			PrintMini( 64,58,(unsigned char*)"Y=",MINI_OVER);
			sprintG(buffer,traceAry[GCursorX],13,LEFT_ALIGN); PrintMini( 72,58,(unsigned char*)buffer,MINI_OVER);
		}
		if ( Derivative ) {
			PrintMini( 64,50,(unsigned char*)"dY/dX=",MINI_OVER);
			dydx = (traceAry[GCursorX+1]-traceAry[GCursorX-1]) / (Xdot*2);
			sprintG(buffer, dydx, 6,LEFT_ALIGN); PrintMini( 88,50,(unsigned char*)buffer,MINI_OVER);
		}
		DrawGCSR(GCursorX,GCursorY); 	// draw graphic cursor
		Bdisp_PutDisp_DD();

		GetKey(&key);
		switch (key) {
			case KEY_CTRL_F1:
			case KEY_CTRL_EXIT:
				cont=0;
				retcode=-1;
				break;
			case KEY_CTRL_F2:
				FkeyZoom();
				GetKey(&key);
				switch (key) {
					case KEY_CTRL_EXIT:
						break;
					case KEY_CTRL_F2:
						cont=0;
						retcode=22;	// zoom fact
						break;
					case KEY_CTRL_F3:
						cont=0;
						retcode=23;	// zoom in
						break;
					case KEY_CTRL_F4:
						cont=0;
						retcode=24;	// zoom out
						break;
					default:
						break;
				}
				break;
			case KEY_CTRL_F3:
				cont=0;
				retcode=3;	// setViewWindow
				break;
			case KEY_CTRL_LEFT:
				GCursorX--;
				if ( GCursorX <   1 )  { cont=0; retcode = GCursorX - 1000; }
				break;
			case KEY_CTRL_RIGHT:
				GCursorX++;
				if ( GCursorX > 127 )  { cont=0; retcode = GCursorX + 1000; }
				break;
			default:
				break;
		}
	}
	GCursorSetFlashMode(0);	// graphic cursor flashing off
	Bkey_Set_RepeatTime(FirstCount,NextCount);	// restore repeat time
	RestoreDisp(SAVEDISP_PAGE1);	// ------ RestoreDisp1
	*index=GCursorX;
	return retcode;	// 0:normal exit
}

//----------------------------------------------------------------------------------------------
//		Setup
//----------------------------------------------------------------------------------------------

void FkeyS_L_(){
	int temp;
	temp=S_L_Style;
	Fkey_Clear( 0 );
	Fkey_dispN( 0, ""); Linesub(0*21+5,7*8+4,0*21+18,7*8+4, S_L_Normal);	// -----
	Fkey_Clear( 1 );
	Fkey_dispN( 1, ""); Linesub(1*21+5,7*8+4,1*21+18,7*8+4, S_L_Thick);	// =====
	Fkey_Clear( 2 );
	Fkey_dispN( 2, ""); Linesub(2*21+5,7*8+4,2*21+18,7*8+4, S_L_Broken);	// : : : :
	Fkey_Clear( 3 );
	Fkey_dispN( 3, ""); Linesub(3*21+5,7*8+4,3*21+18,7*8+4, S_L_Dot);	// . . . .
	Fkey_Clear( 4 );
	Fkey_Clear( 5 );
	S_L_Style=temp;
}

void SetLineStyle() {
	unsigned int key;
	
	FkeyS_L_();
	
	GetKey(&key);
	switch (key) {
		case KEY_CTRL_EXIT:
			break;
		case KEY_CTRL_F1:
			S_L_Style=S_L_Normal;
			break;
		case KEY_CTRL_F2:
			S_L_Style=S_L_Thick ;
			break;
		case KEY_CTRL_F3:
			S_L_Style=S_L_Broken;
			break;
		case KEY_CTRL_F4:
			S_L_Style=S_L_Dot;
			break;
		default:
		break;
	}
}

//-----------------------------------------------------------------------------

void SetVeiwWindowInit(){	// Initialize	retrun 0: no change  -1 : change
		Xmin  =-6.3;
		Xmax  = 6.3;
		Xscl= 1;
		Xdot  = 0.1;
		Ymin  =-3.1;
		Ymax  = 3.1;
		Yscl= 1;
		TThetamin = 0;
		TThetamax = 6.2831853071796;
		TThetaptch= 0.062831853071796;
}
void SetVeiwWindowTrig(){	// trig Initialize
		Xmin  =-9.4247779607694;
		Xmax  = 9.4247779607694;
		Xscl= 1.5707963267949;
		Xdot  = 0.149599650170942;
		Ymin  =-1.6;
		Ymax  = 1.6;
		Yscl= 0.5;
		TThetamin = 0;
		TThetamax = 6.2831853071796;
		TThetaptch= 0.062831853071796;
}
void SetVeiwWindowSTD(){	// STD Initialize
		Xmin  =-10;
		Xmax  = 10;
		Xscl= 1;
		Xdot  =  0.158730158730159;
		Ymin  =-10;
		Ymax  = 10;
		Yscl= 1;
		TThetamin = 0;
		TThetamax = 6.2831853071796;
		TThetaptch= 0.062831853071796;
}


int MathKey( unsigned int  key) {
	switch ( key ) {
			case KEY_CHAR_0:
			case KEY_CHAR_1:
			case KEY_CHAR_2:
			case KEY_CHAR_3:
			case KEY_CHAR_4:
			case KEY_CHAR_5:
			case KEY_CHAR_6:
			case KEY_CHAR_7:
			case KEY_CHAR_8:
			case KEY_CHAR_9:
			case KEY_CHAR_DP:
			case KEY_CHAR_PLUS:
			case KEY_CHAR_MINUS:
			case KEY_CHAR_PMINUS:
			case KEY_CHAR_A:
			case KEY_CHAR_B:
			case KEY_CHAR_C:
			case KEY_CHAR_D:
			case KEY_CHAR_E:
			case KEY_CHAR_F:
			case KEY_CHAR_G:
			case KEY_CHAR_H:
			case KEY_CHAR_I:
			case KEY_CHAR_J:
			case KEY_CHAR_K:
			case KEY_CHAR_L:
			case KEY_CHAR_M:
			case KEY_CHAR_N:
			case KEY_CHAR_O:
			case KEY_CHAR_P:
			case KEY_CHAR_Q:
			case KEY_CHAR_R:
			case KEY_CHAR_S:
			case KEY_CHAR_T:
			case KEY_CHAR_U:
			case KEY_CHAR_V:
			case KEY_CHAR_W:
			case KEY_CHAR_X:
			case KEY_CHAR_Y:
			case KEY_CHAR_Z:
			case KEY_CTRL_XTT:
			case KEY_CHAR_FRAC:   //    0xbb
			case KEY_CHAR_LPAR:   //    0x28
			case KEY_CHAR_LOG:    //    0x95
			case KEY_CHAR_LN:     //    0x85
			case KEY_CHAR_SIN:    //    0x81
			case KEY_CHAR_COS:    //    0x82
			case KEY_CHAR_TAN:    //    0x83
			case KEY_CHAR_SQUARE: //    0x8b
			case KEY_CHAR_POW:    //    0xa8
			case KEY_CHAR_PI:     //    0xd0
			case KEY_CHAR_CUBEROOT: //  0x96
			case KEY_CHAR_EXPN10: //    0xb5
			case KEY_CHAR_EXPN:   //    0xa5
			case KEY_CHAR_ASIN:   //    0x91
			case KEY_CHAR_ACOS:   //    0x92
			case KEY_CHAR_ATAN:   //    0x93
			case KEY_CHAR_ROOT:   //    0x86
			case KEY_CHAR_POWROOT: //   0xb8
				if ( key == KEY_CTRL_XTT ) key='X';
				if ( key == KEY_CHAR_PLUS  )  key='+';
				if ( key == KEY_CHAR_MINUS )  key='-';
				if ( key == KEY_CHAR_PMINUS ) key='-';
				return key;
			default:
				break;
		}
	return 0;
}


int SetViewWindow(void){		// ----------- Set  View Window variable	retrun 0: no change  -1 : change
	char buffer[22];
	unsigned int key;
	int	cont=1;
	int select=0;
	int scrl=0;
	int y;

	double	xmin      = Xmin      ;
	double	xmax      = Xmax      ;
	double	xscl      = Xscl      ;
	double	xdot      = Xdot      ;
	double	ymin      = Ymin      ;
	double	ymax      = Ymax      ;
	double	ydot      = Ydot      ;
	double	yscl      = Yscl      ;
	double	tThetamin = TThetamin ;
	double	tThetamax = TThetamax ;
	double	tThetaptch= TThetaptch;

	while (cont) {
		Bdisp_AllClr_DDVRAM();
		locate( 1,1);Print((unsigned char*)"View Window");

		if ( scrl <=0 ) {
			locate( 1, 2-scrl); Print((unsigned char*)"Xmin  :");
			sprintG(buffer,Xmin,  10,LEFT_ALIGN); locate( 8, 2-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=1 ) {
			locate( 1, 3-scrl); Print((unsigned char*)" max  :");
			sprintG(buffer,Xmax,  10,LEFT_ALIGN); locate( 8, 3-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=2 ) {
			locate( 1, 4-scrl); Print((unsigned char*)" scale:");
			sprintG(buffer,Xscl,10,LEFT_ALIGN); locate( 8, 4-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=3 ) {
			locate( 1, 5-scrl); Print((unsigned char*)" dot  :");
			sprintG(buffer,Xdot,  10,LEFT_ALIGN); locate( 8, 5-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=4 ) {
			locate( 1, 6-scrl); Print((unsigned char*)"Ymin  :");
			sprintG(buffer,Ymin,  10,LEFT_ALIGN); locate( 8, 6-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=5 ) {
			locate( 1, 7-scrl); Print((unsigned char*)" max  :");
			sprintG(buffer,Ymax,  10,LEFT_ALIGN); locate( 8, 7-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl >=1 ) {
			locate( 1, 8-scrl); Print((unsigned char*)" scale:");
			sprintG(buffer,Yscl,10,LEFT_ALIGN); locate( 8, 8-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl >=2 ) {
			locate( 1, 9-scrl); Print((unsigned char*)"Temin :");
			sprintG(buffer,TThetamin,  10,LEFT_ALIGN); locate( 8, 9-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl >=3 ) {
			locate( 1, 10-scrl); Print((unsigned char*)"  max :");
			sprintG(buffer,TThetamax,  10,LEFT_ALIGN); locate( 8, 10-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl >=4 ) {
			locate( 1, 11-scrl); Print((unsigned char*)"  ptch:");
			sprintG(buffer,TThetaptch,  10,LEFT_ALIGN); locate( 8, 11-scrl); Print((unsigned char*)buffer);
		}

		y = select-scrl+1;
		Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
		
		Fkey_dispN(0,"Init");
		Fkey_dispN(1,"Trig");
		Fkey_dispN(2,"STD");

		Bdisp_PutDisp_DD();

		GetKey( &key );
		switch (key) {
			case KEY_CTRL_EXIT:
				return;
				break;
			case KEY_CTRL_EXE:
				cont=0;
				break;
		
			case KEY_CTRL_UP:
				select-=1;
				if ( select < 0 ) {select=9; scrl=4;}
				if ( select < scrl ) scrl-=1;
				if ( scrl < 0 ) scrl=0;
				break;
			case KEY_CTRL_DOWN:
				select+=1;
				if ( select > 9 ) {select=0; scrl=0;}
				if ((select - scrl) > 5 ) scrl+=1;
				if ( scrl > 4 ) scrl=4;
				break;
				
			case KEY_CTRL_F1:	// Initialize
				SetVeiwWindowInit();
				break;
			case KEY_CTRL_F2:	// trig Initialize
				SetVeiwWindowTrig();
				break;
			case KEY_CTRL_F3:	// STD Initialize
				SetVeiwWindowSTD();
				break;
			
			case KEY_CTRL_RIGHT:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				Fkey_Clear( 0 );
				Fkey_Clear( 1 );
				Fkey_Clear( 2 );
				y++;
				switch (select) {
					case 0: // Xmin
						Xmin      =InputNumD_full( 8, y, 14, Xmin);	// 
						break;
					case 1: // Xmax
						Xmax      =InputNumD_full( 8, y, 14, Xmax);	// 
						break;
					case 2: // Xscl
						Xscl      =InputNumD_full( 8, y, 14, Xscl);	// 
						break;
					case 3: // Xdot
						Xdot      =InputNumD_full( 8, y, 14, Xdot);	// 
						break;
					case 4: // Ymin
						Ymin      =InputNumD_full( 8, y, 13, Ymin);	// 
						break;
					case 5: // Ymax
						Ymax      =InputNumD_full( 8, y, 13, Ymax);	// 
						break;
					case 6: // Yscl
						Yscl      =InputNumD_full( 8, y, 13, Yscl);	// 
						break;
					case 7: // TThetamin
						TThetamin =InputNumD_full( 8, y, 13, TThetamin);	// 
						break;
					case 8: // TThetamax
						TThetamax =InputNumD_full( 8, y, 13, TThetamax);	// 
						break;
					case 9: // TThetaptch
						TThetaptch=InputNumD_full( 8, y, 13, TThetaptch);	// 
						break;
					default:
						break;
				}
				break;
			default:
				break;
		}
		if ( MathKey( key ) ) {
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				Fkey_Clear( 0 );
				Fkey_Clear( 1 );
				Fkey_Clear( 2 );
				y++;
				switch (select) {
					case 0: // Xmin
						Xmin      =InputNumD_Char( 8, y, 14, Xmin, key);	// 
						break;
					case 1: // Xmax
						Xmax      =InputNumD_Char( 8, y, 14, Xmax, key);	// 
						break;
					case 2: // Xscl
						Xscl      =InputNumD_Char( 8, y, 14, Xscl, key);	// 
						break;
					case 3: // Xdot
						Xdot      =InputNumD_Char( 8, y, 14, Xdot, key);	// 
						break;
					case 4: // Ymin
						Ymin      =InputNumD_Char( 8, y, 13, Ymin, key);	// 
						break;
					case 5: // Ymax
						Ymax      =InputNumD_Char( 8, y, 13, Ymax, key);	// 
						break;
					case 6: // Yscl
						Yscl      =InputNumD_Char( 8, y, 13, Yscl, key);	// 
						break;
					case 7: // TThetamin
						TThetamin =InputNumD_Char( 8, y, 13, TThetamin, key);	// 
						break;
					case 8: // TThetamax
						TThetamax =InputNumD_Char( 8, y, 13, TThetamax, key);	// 
						break;
					case 9: // TThetaptch
						TThetaptch=InputNumD_Char( 8, y, 13, TThetaptch, key);	// 
						break;
					default:
						break;
				}
			}
	}

	if( ( xmin      != Xmin      ) ||
	    ( xmax      != Xmax      ) ||
	    ( xscl      != Xscl      ) ||
	    ( xdot      != Xdot      ) ||
	    ( ymin      != Ymin      ) ||
	    ( ymax      != Ymax      ) ||
	    ( ydot      != Ydot      ) ||
	    ( yscl      != Yscl      ) ||
	    ( tThetamin != TThetamin ) ||
	    ( tThetamax != TThetamax ) ||
	    ( tThetaptch!= TThetaptch) )  {
			ViewWindow( Xmin, Xmax, Xscl, Ymin, Ymax, Yscl);
			return -1; // change value
	}
	return 0;	// no change
}

//-----------------------------------------------------------------------------
void SetFactor(){
	char buffer[22];
	unsigned int key;
	int	cont=1;
	int select=0;
	int y;

	PopUpWin(3);

	while (cont) {
		locate(3,3); Print((unsigned char *)"Factor");
		locate(3,4); Print((unsigned char *) "Xfct:           ");
		sprintG(buffer,Xfct,  10,LEFT_ALIGN); locate( 9, 4); Print((unsigned char*)buffer);
		locate(3,5); Print((unsigned char *) "Yfct:           ");
		sprintG(buffer,Yfct,  10,LEFT_ALIGN); locate( 9, 5); Print((unsigned char*)buffer);

		y = select + 3 ;
		Bdisp_AreaReverseVRAM(12, y*8, 113, y*8+7);	// reverse select line 
		Bdisp_PutDisp_DD();

		GetKey( &key );
		switch (key) {
			case KEY_CTRL_EXIT:
			case KEY_CTRL_EXE:
				cont=0;
				break;
		
			case KEY_CTRL_UP:
				select-=1;
				if ( select < 0 ) select=1;
				break;
			case KEY_CTRL_DOWN:
				select+=1;
				if ( select > 1 ) select=0;
				break;

			case KEY_CTRL_RIGHT:
				Bdisp_AreaReverseVRAM(12, y*8, 113, y*8+7);	// reverse select line 
				Bdisp_PutDisp_DD();
				y++;
				switch (select) {
					case 0: // Xfct
						Xfct  =InputNumD_full( 9, y, 10, Xfct);	// 
						break;
					case 1: // Yfct
						Xmax   =InputNumD_full( 9, y, 10, Yfct);	// 
						break;
					default:
						break;
				}
				break;
			default:
				break;
		}
		if ( MathKey( key ) ) {
				Bdisp_AreaReverseVRAM(12, y*8, 113, y*8+7);	// reverse select line 
				Bdisp_PutDisp_DD();
				y++;
				switch (select) {
					case 0: // Xfct
						Xfct  =InputNumD_Char( 9, y, 10, Xfct, key);	// 
						break;
					case 1: // Yfct
						Yfct  =InputNumD_Char( 9, y, 10, Yfct, key);	// 
						break;
					default:
						break;
				}
		}

	}
}

//-----------------------------------------------------------------------------

void SetVar(int select){		// ----------- Set Variable
	char buffer[22];
	unsigned int key;
	int	cont=1;
	int scrl=0;
	int y;
	int selectreplay=-1;
	
	scrl=select;
	if ( scrl > 19 ) scrl=19;
	
	while (cont) {
		Bdisp_AllClr_DDVRAM();

		if ( scrl <=0 ) {
			locate( 1, 1-scrl); Print((unsigned char*)"A=");
			sprintG(buffer, regA, 19,LEFT_ALIGN); locate( 3, 1-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=1 ) {
			locate( 1, 2-scrl); Print((unsigned char*)"B=");
			sprintG(buffer, regB, 19,LEFT_ALIGN); locate( 3, 2-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=2 ) {
			locate( 1, 3-scrl); Print((unsigned char*)"C=");
			sprintG(buffer, regC, 19,LEFT_ALIGN); locate( 3, 3-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=3 ) {
			locate( 1, 4-scrl); Print((unsigned char*)"D=");
			sprintG(buffer, regD, 19,LEFT_ALIGN); locate( 3, 4-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=4 ) {
			locate( 1, 5-scrl); Print((unsigned char*)"E=");
			sprintG(buffer, regE, 19,LEFT_ALIGN); locate( 3, 5-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=5 ) {
			locate( 1, 6-scrl); Print((unsigned char*)"F=");
			sprintG(buffer, regF, 19,LEFT_ALIGN); locate( 3, 6-scrl); Print((unsigned char*)buffer);
		}
		if ( scrl <=6 ) {
			locate( 1, 7-scrl); Print((unsigned char*)"G=");
			sprintG(buffer, regG, 19,LEFT_ALIGN); locate( 3, 7-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=1 ) && ( 8-scrl > 0 ) ){
			locate( 1, 8-scrl); Print((unsigned char*)"H=");
			sprintG(buffer, regH, 19,LEFT_ALIGN); locate( 3, 8-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=2 ) && ( 9-scrl > 0 ) ) {
			locate( 1, 9-scrl); Print((unsigned char*)"I=");
			sprintG(buffer, regI, 19,LEFT_ALIGN); locate( 3, 9-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=3 ) && ( 10-scrl > 0 ) ) {
			locate( 1, 10-scrl); Print((unsigned char*)"J=");
			sprintG(buffer, regJ, 19,LEFT_ALIGN); locate( 3, 10-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=4 ) && ( 11-scrl > 0 ) ) {
			locate( 1, 11-scrl); Print((unsigned char*)"K=");
			sprintG(buffer, regK, 19,LEFT_ALIGN); locate( 3, 11-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=5 ) && ( 12-scrl > 0 ) ) {
			locate( 1, 12-scrl); Print((unsigned char*)"L=");
			sprintG(buffer, regL, 19,LEFT_ALIGN); locate( 3, 12-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=6 )  && ( 13-scrl > 0 ) ) {
			locate( 1, 13-scrl); Print((unsigned char*)"M=");
			sprintG(buffer, regM, 19,LEFT_ALIGN); locate( 3, 13-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=7 ) && ( 14-scrl > 0 ) ) {
			locate( 1, 14-scrl); Print((unsigned char*)"N=");
			sprintG(buffer, regN, 19,LEFT_ALIGN); locate( 3, 14-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=8 ) && ( 15-scrl > 0 ) ) {
			locate( 1, 15-scrl); Print((unsigned char*)"O=");
			sprintG(buffer, regO, 19,LEFT_ALIGN); locate( 3, 15-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=9 )  && ( 16-scrl > 0 ) ) {
			locate( 1, 16-scrl); Print((unsigned char*)"P=");
			sprintG(buffer, regP, 19,LEFT_ALIGN); locate( 3, 16-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=10 ) && ( 17-scrl > 0 ) ) {
			locate( 1, 17-scrl); Print((unsigned char*)"Q=");
			sprintG(buffer, regQ, 19,LEFT_ALIGN); locate( 3, 17-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=11 ) && ( 18-scrl > 0 ) ) {
			locate( 1, 18-scrl); Print((unsigned char*)"R=");
			sprintG(buffer, regR, 19,LEFT_ALIGN); locate( 3, 18-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=12 ) && ( 19-scrl > 0 ) ) {
			locate( 1, 19-scrl); Print((unsigned char*)"S=");
			sprintG(buffer, regS, 19,LEFT_ALIGN); locate( 3, 19-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=13 ) && ( 20-scrl > 0 ) ) {
			locate( 1, 20-scrl); Print((unsigned char*)"T=");
			sprintG(buffer, regT, 19,LEFT_ALIGN); locate( 3, 20-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=14 ) && ( 21-scrl > 0 ) ) {
			locate( 1, 21-scrl); Print((unsigned char*)"U=");
			sprintG(buffer, regU, 19,LEFT_ALIGN); locate( 3, 21-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=15 ) && ( 22-scrl > 0 ) ) {
			locate( 1, 22-scrl); Print((unsigned char*)"V=");
			sprintG(buffer, regV, 19,LEFT_ALIGN); locate( 3, 22-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=16 ) && ( 23-scrl > 0 ) ) {
			locate( 1, 23-scrl); Print((unsigned char*)"W=");
			sprintG(buffer, regW, 19,LEFT_ALIGN); locate( 3, 23-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=17 ) && ( 24-scrl > 0 ) ) {
			locate( 1, 24-scrl); Print((unsigned char*)"X=");
			sprintG(buffer, regX, 19,LEFT_ALIGN); locate( 3, 24-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=18 ) && ( 25-scrl > 0 ) ) {
			locate( 1, 25-scrl); Print((unsigned char*)"Y=");
			sprintG(buffer, regY, 19,LEFT_ALIGN); locate( 3, 25-scrl); Print((unsigned char*)buffer);
		}
		if ( ( scrl >=19 ) && ( 26-scrl > 0 ) ) {
			locate( 1, 26-scrl); Print((unsigned char*)"Z=");
			sprintG(buffer, regZ, 19,LEFT_ALIGN); locate( 3, 26-scrl); Print((unsigned char*)buffer);
		}

		y = select-scrl;
		Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
		
		Bdisp_PutDisp_DD();

		GetKey( &key );
		switch (key) {
			case KEY_CTRL_EXIT:
			case KEY_CTRL_EXE:
				cont=0;
				break;
		
			case KEY_CTRL_UP:
				select-=1;
				if ( select < 0 ) {select=25; scrl=19;}
				if ( select < scrl ) scrl-=1;
				if ( scrl < 0 ) scrl=0;
				selectreplay = -1; // replay cancel
				break;
			case KEY_CTRL_DOWN:
				select+=1;
				if ( select > 25 ) {select=0; scrl=0;}
				if ((select - scrl) > 6 ) scrl+=1;
				if ( scrl > 19 ) scrl=19;
				selectreplay = -1; // replay cancel
				break;
				
			case KEY_CTRL_RIGHT:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				Fkey_Clear( 0 );
				Fkey_Clear( 1 );
				Fkey_Clear( 2 );
				y++;
				selectreplay = select; 
				if ( ( 0 <= select ) && ( select <=25 ) ) {	// regA to regZ
						REG[select]= InputNumD_full( 3, y, 19, REG[select]);
				} else {
						selectreplay = -1; // replay cancel 
				}
				break;
				
			case KEY_CTRL_LEFT:
				if (selectreplay<0) break;
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				Fkey_Clear( 0 );
				Fkey_Clear( 1 );
				Fkey_Clear( 2 );
				y++;
				selectreplay = select; 
				if ( ( 0 <= select ) && ( select <=25 ) ) {	// regA to regZ
						REG[select]= InputNumD_replay( 3, y, 19, REG[select]);
				} else {
						selectreplay = -1; // replay cancel 
				}
				break;
			default:
				break;
		}
		if ( MathKey( key ) ) {
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				Fkey_Clear( 0 );
				Fkey_Clear( 1 );
				Fkey_Clear( 2 );
				y++;
				selectreplay = select; 
				if ( ( 0 <= select ) && ( select <=25 ) ) {	// regA to regZ
						REG[select]= InputNumD_Char( 3, y, 19, REG[select], key);
				} else {
						selectreplay = -1; // replay cancel 
				}
		}
	}
}

//-----------------------------------------------------------------------------

void SetupG(){		// ----------- Setup graphic
    char *onoff[]  ={"off","on"};
    char *draw[]   ={"Connect","Plot"};
    char *style[]  ={"Normal","Thick","Broken","Dot"};
	char buffer[22];
	unsigned int key;
	int	cont=1;
	int select=0;
	int scrl=0;
	int y;

	while (cont) {
		Bdisp_AllClr_DDVRAM();
		if ( scrl <=0 ) {
			locate( 1, 1-scrl); Print((unsigned char*)"Draw Type   :");
			locate(14, 1-scrl); Print((unsigned char*)draw[(int)DrawType]);
		}
		if ( scrl <=1 ) {
			locate( 1, 2-scrl); Print((unsigned char*)"Coord:      :");
			locate(14, 2-scrl); Print((unsigned char*)onoff[Coord]);
		}
		if ( scrl <=2 ) {
			locate( 1, 3-scrl); Print((unsigned char*)"Grid        :");
			locate(14, 3-scrl); Print((unsigned char*)onoff[Grid]);
		}
		if ( scrl <=3 ) {
			locate( 1, 4-scrl); Print((unsigned char*)"Axes        :");
			locate(14, 4-scrl); Print((unsigned char*)onoff[Axes]);
		}
		if ( scrl <=4 ) {
			locate( 1, 5-scrl); Print((unsigned char*)"Label       :");
			locate(14, 5-scrl); Print((unsigned char*)onoff[Label]);
		}
		if ( scrl <=5 ) {
			locate( 1, 6-scrl); Print((unsigned char*)"Derivative  :");
			locate(14, 6-scrl); Print((unsigned char*)onoff[Derivative]);
		}
		if ( scrl <=6 ) {
			locate( 1, 7-scrl); Print((unsigned char*)"Sketch Line :");
			locate(14, 7-scrl); Print((unsigned char*)style[S_L_Style]);
		}

		y = select-scrl;
		Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
		switch (y) {
			case 0: // Draw Type
				Fkey_dispN( 0, "Con");
				Fkey_dispN( 1, "Plot");
				Fkey_Clear( 2 );
				Fkey_Clear( 3 );
				break;
			case 1: // Coord
			case 2: // Grid	
			case 3: // Axes
			case 4: // Label
			case 5: // Derivative
				Fkey_dispN( 0, " On ");
				Fkey_dispN( 1, " Off");
				Fkey_Clear( 2 );
				Fkey_Clear( 3 );
				break;
			case 6: // S_L_ Line	normal
				FkeyS_L_();
				break;
			default:
				break;
		}
			
		
		Bdisp_PutDisp_DD();

		GetKey( &key );
		switch (key) {
			case KEY_CTRL_EXIT:
			case KEY_CTRL_EXE:
				cont=0;
				break;
		
			case KEY_CTRL_UP:
				select-=1;
				if ( select < 0 ) {select=6; scrl=0;}
//				if ( select < scrl ) scrl-=1;
//				if ( scrl < 0 ) scrl=0;
				break;
			case KEY_CTRL_DOWN:
				select+=1;
				if ( select > 6 ) {select=0; scrl=0;}
//				if ((select - scrl) > 6 ) scrl+=1;
//				if ( scrl > 0 ) scrl=0;
				break;
				
			case KEY_CTRL_F1:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				switch (select) {
					case 0: // Draw Type connect
						DrawType = 0 ;
						break;
					case 1: // Coord	on
						Coord = 1 ;
						break;
					case 2: // Grid		on
						Grid = 1 ;
						break;
					case 3: // Axes		on
						Axes = 1 ;
						break;
					case 4: // Label	on
						Label = 1 ;
						break;
					case 5: // 	Derivative on
						Derivative = 1 ;
						break;
					case 6: // Sketch Line	normal
						S_L_Style = 0 ;
						break;
					default:
						break;
				}
				break;
			case KEY_CTRL_F2:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				switch (select) {
					case 0: // Draw Type Plot
						DrawType = 1 ;
						break;
					case 1: // Coord	off
						Coord = 0 ;
						break;
					case 2: // Grid		off
						Grid = 0 ;
						break;
					case 3: // Axes		off
						Axes = 0 ;
						break;
					case 4: // Label	off
						Label = 0 ;
						break;
					case 5: // 	Derivative off
						Derivative = 0 ;
						break;
					case 6: // Sketch Line	Thick
						S_L_Style = 1 ; 
						break;
					default:
						break;
				}
				break;
			case KEY_CTRL_F3:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				switch (select) {
					case 6: // Sketch Line	Broken
						S_L_Style = 2 ; 
						break;
					default:
						break;
				}
				break;
			case KEY_CTRL_F4:
				Bdisp_AreaReverseVRAM(0, y*8, 127, y*8+7);	// reverse select line 
				switch (select) {
					case 6: // Sketch  Line	Dot
						S_L_Style = 3 ; 
						break;
					default:
						break;
				}
				break;
			default:
				break;
		}
	}
}

//----------------------------------------------------------------------------------------------
//		Expression evaluation    string -> double
//----------------------------------------------------------------------------------------------
#define ExpMax 255
char ExpBuffer[ExpMax+1];
int ExpPtr;	// Expression buffer ptr
int ErrPtr;	// Error ptr
//-----------------------------------------------------------------------------

double InputNumD_full(int x, int y, int width, double defaultNum) {		// full number display
	double result;
	sprintG(ExpBuffer, defaultNum, 30, LEFT_ALIGN);
	if ( InputStrSub( x, y, width, 0, ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
	result = Eval( ExpBuffer );
	while ( ErrPtr ) {	// error loop
		if ( InputStrSub( x, y, width, ErrPtr, ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
		result = Eval( ExpBuffer );
	}
	return result; // value ok
}

double InputNumD_Char(int x, int y, int width, double defaultNum, char code) {		//  1st char key in
	double result;
	ExpBuffer[0]=code;
	ExpBuffer[1]='\0';
	if ( InputStrSub( x, y, width, 1, ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
	result = Eval( ExpBuffer );
	while ( ErrPtr ) {	// error loop
		if ( InputStrSub( x, y, width, ErrPtr, ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
		result = Eval( ExpBuffer );
	}
	return result; // value ok
}

double InputNumD_replay(int x, int y, int width, double defaultNum) {		//  replay expression
	double result;
	if ( InputStrSub( x, y, width, strlen(ExpBuffer), ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
	result = Eval( ExpBuffer );
	while ( ErrPtr ) {	// error loop
		if ( InputStrSub( x, y, width, ErrPtr, ExpBuffer, 64, ' ', REV_OFF, FLOAT_ON, EXP_ON, ALPHA_ON) ) return (defaultNum);
		result = Eval( ExpBuffer );
	}
	return result; // value ok
}

//-----------------------------------------------------------------------------

void ConvertBuffer( char *dist, char *src, int length) {
	int ptr=0;
	unsigned char c;
	while ( length ) {
		c=src[ptr];
		if ( c == 0x87  )  c='-'; // (-)
		if ( c == 0x0F  )  c='e'; // exp
		dist[ptr]=c; ptr++;
		length--;
	}
	dist[ptr]='\0';
}

void EvalERR(char *buffer, int err) {
	unsigned int key;
	char buf[20];

	if ( ErrPtr ) return ;
	SaveDisp(SAVEDISP_PAGE1);
	PopUpWin(5);
	locate(3,2); Print((unsigned char *)buffer);
	sprintf(buf,"code:%d",err);
	locate(3,4); Print((unsigned char *)buf);
	locate(3,6); Print((unsigned char *) "   Press:[EXIT]");
	Bdisp_PutDisp_DD();

	GetKey(&key);	
	RestoreDisp(SAVEDISP_PAGE1);
	Bdisp_PutDisp_DD();
	
	ErrPtr=ExpPtr;	// error Ptr
}

int SkipNum(char *expbuf){
	unsigned char c;
	c = expbuf[ExpPtr];
	while ( ('0'<=c)&&(c<='9') ) c=expbuf[++ExpPtr];
	return c;
}

void NextToken(char *expbuf) {
	unsigned char c;
	c = expbuf[ExpPtr];
	if ( c == '.'  )   {
			ExpPtr++;
			c=SkipNum(expbuf);
	} else { 
			c=SkipNum(expbuf);
			if ( c == '.'  ) {
				ExpPtr++;
				c=SkipNum(expbuf);
			}
	}
	if ( ( c == 'e'  ) || ( c == 'E' ) ) { 
			c=expbuf[++ExpPtr];
			if ( ( c == '-'  ) || ( c == '+' ) ) c=expbuf[++ExpPtr];	// - +
			if ( ( '0'<=c )&&( c<='9' ) ) c=SkipNum(expbuf);
			else EvalERR("Syntax Error",ExpPtr); // error 
		
	}
}

double EvalGetNum(char *expbuf){
	double result,tmp;
	unsigned char c;
	c = expbuf[ExpPtr];
	if ( c == '+' ) while ( '+'==c ) c=expbuf[++ExpPtr];
	if ( c == '-' ) {	//  -
		ExpPtr++;
		return - EvalGetNum( expbuf );
	}
	if ( ( 'A'<=c )&&( c<='Z' ) ) {
			ExpPtr++ ;
			return REG[c-'A'] ;
	}
	if ( c==0xD0 ) { // PI
			ExpPtr++;
			return PI ;
	}
	if ( ( c=='.' ) || ( ( '0'<=c )&&( c<='9' ) ) ) {
			result = atof( expbuf+ExpPtr );
			NextToken( expbuf );
			return result ;
	}
	switch ( c ) { 			// ( type C function )  sin cos tan... 
		case 0x86 :	// sqr
			ExpPtr++; result = sqrt( Evalsub7( expbuf ) );
			return result ;
		case 0x81 :	// sin
			ExpPtr++; result = sin( Evalsub7( expbuf ) );
			return result ;
		case 0x82 :	// cos
			ExpPtr++; result = cos( Evalsub7( expbuf ) );
			return result ;
		case 0x83 :	// tan
			ExpPtr++; result = tan( Evalsub7( expbuf ) );
			return result ;
		case 0x91 :	// asin
			ExpPtr++; result = asin( Evalsub7( expbuf ) );
			return result ;
		case 0x92 :	// acos
			ExpPtr++; result = acos( Evalsub7( expbuf ) );
			return result ;
		case 0x93 :	// atan
			ExpPtr++; result = atan( Evalsub7( expbuf ) );
			return result ;
		case 0x85 :	// ln
			ExpPtr++; result = log( Evalsub7( expbuf ) );
			return result ;
		case 0x95 :	// log10
			ExpPtr++; result = log10( Evalsub7( expbuf ) );
			return result ;
		case 0x96 :	// cuberoot
			ExpPtr++; result = pow( Evalsub7( expbuf ), 1.0/3.0 );
			return result ;
		case 0x97 :	// abs
			ExpPtr++; result = abs( Evalsub7( expbuf ) );
			return result ;
		case 0xA5 :	// expn
			ExpPtr++; result = exp( Evalsub7( expbuf ) );
			return result ;
		case 0xA6 :	// int
			ExpPtr++; result = floor( Evalsub7( expbuf ) );
			return result ;
		case 0xA7 :	// Not
			ExpPtr++; result = ! (int) ( Evalsub7( expbuf ) );
			return result ;
		case 0xB5 :	// 10^
			ExpPtr++; result = pow(10, Evalsub7( expbuf ) );
			return result ;
		case 0xB6 :	// frac
			ExpPtr++;
			tmp = floor ( Evalsub7( expbuf ) );
			return tmp- floor(tmp) ;
		default:
			break;
	}
	EvalERR("Syntax Error",ExpPtr); // error 
	return 0 ;
}


double Evalsub0(char *expbuf) {	// 0th Priority
	double result;
	unsigned char c;
	c = expbuf[ExpPtr];
	if ( c == '(') {
		ExpPtr++;
		result = EvalsubTop( expbuf );
		if ( expbuf[ExpPtr] == ')' ) ExpPtr++;
		return result;
	 } else 
	 	return EvalGetNum( expbuf );
}

double Evalsub1(char *expbuf) {	//  1th Priority  ( type A function )
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub0( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case 0x8E :		// Mod( , )
				ExpPtr++;
				tmp = EvalsubTop( expbuf );
				if ( tmp == 0 ) EvalERR("Division by 0 ",ExpPtr); // error 
				if ( expbuf[ExpPtr] != ',' ) EvalERR("Syntax Error",ExpPtr); // error 
				ExpPtr++;
				result= fmod( tmp, EvalsubTop( expbuf ) ) ;
				if ( expbuf[ExpPtr] == ')' ) ExpPtr++;
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}
double Evalsub2(char *expbuf) {	//  2nd Priority  ( type B function ) ...
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub1( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case  '!'  :	//  !
				ExpPtr++;
				tmp = floor( result );
				result = 1;
				while ( tmp > 0 ) { result *= tmp; tmp--; }
				break;
			case  0x8B  :	// ^2
				ExpPtr++;
				result *= result ;
				break;
			case  0x9B  :	// ^(-1) RECIP
				ExpPtr++;
				result = 1 / result ;
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}
double Evalsub3(char *expbuf) {	//  3rd Priority  ( ^ ...)
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub2( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case  0xA8  :	// a ^ b
			case  '^'  :
				ExpPtr++;
				result = pow( result, Evalsub2( expbuf ) );
				break;
			case  0xB8  :	// powroot
				ExpPtr++;
				if ( result == 0 ) EvalERR("Math error ",ExpPtr); // error 
				result = pow( Evalsub2( expbuf ), 1/result );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}
double Evalsub4(char *expbuf) {	//  4th Priority  (Fraction) a/b/c
	double result,frac1,frac2,frac3;
	unsigned char c;
	result = Evalsub3( expbuf );
	c = expbuf[ExpPtr];
	if ( c == 0xBB ) {
		ExpPtr++;
		frac1 = result ;
		frac2 = Evalsub3( expbuf );
		c = expbuf[ExpPtr];
		if ( c == 0xBB ) {
			ExpPtr++;
			frac3 = Evalsub3( expbuf );
			if ( frac3 == 0 ) EvalERR("Math error ",ExpPtr); // error 
			result = frac1 + ( frac2 / frac3 ) ;
		} else {
			if ( frac2 == 0 ) EvalERR("Math error ",ExpPtr); // error 
			result = ( frac1 / frac2 ) ;
		}
	}
	return result;
}
double Evalsub5(char *expbuf) {	//  5th Priority
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub4( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		if ( ( 'A' <= c ) && ( c <= 'Z' ) ) {
			result *= Evalsub4( expbuf ) ;
		} else if ( c == 0xD0 ) { // PI
			result *= Evalsub4( expbuf ) ;
		} else 
			cont=0;
	 }
	return result;
}
double Evalsub6(char *expbuf) {	//  6th Priority  ( type C function )  sin cos tan... 
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub5( expbuf );
	return result;
}
double Evalsub7(char *expbuf) {	//  7th Priority
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub6( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case '(' :
				ExpPtr++; result *= EvalsubTop( expbuf );
				if ( expbuf[ExpPtr] == ')' ) ExpPtr++;
				break;
			case 0x81 :	// sin
				ExpPtr++; result *= sin( Evalsub6( expbuf ) );
				break;
			case 0x82 :	// cos
				ExpPtr++; result *= cos( Evalsub6( expbuf ) );
				break;
			case 0x83 :	// tan
				ExpPtr++; result *= tan( Evalsub6( expbuf ) );
				break;
			case 0x91 :	// asin
				ExpPtr++; result *= asin( Evalsub6( expbuf ) );
				break;
			case 0x92 :	// acos
				ExpPtr++; result *= acos( Evalsub6( expbuf ) );
				break;
			case 0x93 :	// atan
				ExpPtr++; result *= atan( Evalsub6( expbuf ) );
				break;
			case 0x85 :	// ln
				ExpPtr++; result *= log( Evalsub6( expbuf ) );
				break;
			case 0x86 :	// sqr
				ExpPtr++; result *= sqrt( Evalsub6( expbuf ) );
				break;
			case 0x95 :	// log10
				ExpPtr++; result *= log10( Evalsub6( expbuf ) );
				break;
			case 0x96 :	// cuberoot
				ExpPtr++; result *= pow( Evalsub6( expbuf ), 1.0/3.0 );
				break;
			case 0x97 :	// abs
				ExpPtr++; result *= fabs( Evalsub6( expbuf ) );
				break;
			case 0xA5 :	// expn
				ExpPtr++; result *= exp( Evalsub6( expbuf ) );
				break;
			case 0xA6 :	// int
				ExpPtr++; result *= floor( Evalsub6( expbuf ) );
				break;
			case 0xA7 :	// Not
				ExpPtr++; result *= ! (int)( Evalsub6( expbuf ) );
				break;
			case 0xB5 :	// 10^
				ExpPtr++; result *= pow( 10, Evalsub6( expbuf ) );
				break;
			case 0xB6 :	// frac
				ExpPtr++; result *= floor( Evalsub6( expbuf ) );
				break;
			case 0xB7 :	// Neg
				ExpPtr++; result *= - ( Evalsub6( expbuf ) );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}
double Evalsub8(char *expbuf) {	//  8th Priority
	double result,tmp;
	unsigned char c;
	result = Evalsub7( expbuf );
	return result;
}
double Evalsub9(char *expbuf) {	//  9th Priority
	double result,tmp;
	unsigned char c;
	result = Evalsub8( expbuf );
	return result;
}
double Evalsub10(char *expbuf) {	//  10th Priority  ( *,/, int.,Rmdr )
	int cont=1;
	double result,tmp;
	unsigned char c;
	result = Evalsub9( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case  0xA9 :	// ~
				ExpPtr++;
				result *= Evalsub9( expbuf );
				break;
			case 0xB9 :		// 
				ExpPtr++;
				tmp = Evalsub9( expbuf );
				if ( tmp == 0 ) EvalERR("Division by 0 ",ExpPtr); // error 
				result /= tmp ;
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}
 
double Evalsub11(char *expbuf) {	//  11th Priority  ( +,- )
	int cont=1;
	double result;
	unsigned char c;
	result = Evalsub10( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case '+' :
				ExpPtr++;
				result += Evalsub10( expbuf );
				break;
			case '-' :
				ExpPtr++;
				result -= Evalsub10( expbuf );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}

double Evalsub12(char *expbuf) {	//  12th Priority ( =,!=,><,>=,<= )
	int cont=1;
	double result;
	unsigned char c;
	result = Evalsub11( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case '=' :	// =
				ExpPtr++;
				result = ( result == Evalsub11( expbuf ) );
				break;
			case '>' :	// >
				ExpPtr++;
				result = ( result >  Evalsub11( expbuf ) );
				break;
			case '<' :	// <
				ExpPtr++;
				result = ( result <  Evalsub11( expbuf ) );
				break;
			case 0x11 :	// !=
				ExpPtr++;
				result = ( result != Evalsub11( expbuf ) );
				break;
			case 0x12 :	// >=
				ExpPtr++;
				result = ( result >= Evalsub11( expbuf ) );
				break;
			case 0x10 :	// <=
				ExpPtr++;
				result = ( result <= Evalsub11( expbuf ) );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}

double Evalsub13(char *expbuf) {	//  13th Priority  ( And,and)
	int cont=1;
	double result;
	unsigned char c;
	result = Evalsub12( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case 0xBA :	// And
				ExpPtr++;
				result = ( (int)result & (int)Evalsub13( expbuf ) );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}

double EvalsubTop(char *expbuf) {	//  14th Priority  ( Or,Xor,or,xor,xnor )
	int cont=1;
	double result;
	unsigned char c;
	result = Evalsub13( expbuf );
	while ( cont ) {
		c = expbuf[ExpPtr];
		switch ( c ) {
			case 0xAA :	// Or
				ExpPtr++;
				result = ( (int)result | (int)Evalsub13( expbuf ) );
				break;
			case 0x9A :	// Xor
				ExpPtr++;
				result = ( (int)result ^ (int)Evalsub13( expbuf ) );
				break;
			case 0x8A :	// Xnor
				ExpPtr++;
				result = !( (int)result ^ (int)Evalsub13( expbuf ) );
				break;
			default:
				cont=0;
				break;
		}
	 }
	return result;
}

double Eval(char *expbuf) {		// Eval
	unsigned int key;
	double result;
    ExpPtr= 0;
    ErrPtr= 0;
	if ( strlen(expbuf) == 0 ) return 0;
    result = EvalsubTop( expbuf );
    if ( ExpPtr < strlen(expbuf) ) 	EvalERR("Syntax Error",ExpPtr); // error 
	return result;
}

